/* A ceelog expression.

Copyright (C) 2012 Red Hat, Inc. All rights reserved.
This copyrighted material is made available to anyone wishing to use, modify,
copy, or redistribute it subject to the terms and conditions of the GNU General
Public License v.2.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301, USA.

Author: Miloslav Trmač <mitr@redhat.com> */

#ifndef CEELOG_EXPRESSION_H__
#define CEELOG_EXPRESSION_H__

#include <stdbool.h>

#include <glib.h>

G_BEGIN_DECLS

 /* Expression creation */

/* A ceelog expression.  Currently expressions can only have a boolean result,
   but that might change in the future. */
struct ceelog_expr;

/* Free EXPR, including all its subexpressions. */
extern void ceelog_expr_free (struct ceelog_expr *expr);

/* A comparison operation */
enum ceelog_expr_op
  {
    CEELOG_EO_EQ, CEELOG_EO_NE,
    CEELOG_EO_MATCHES, CEELOG_EO_DOESNT_MATCH,
    CEELOG_EO_AND, CEELOG_EO_OR, CEELOG_EO_NOT,
  };

/* Create a comparison expression that compares FIELD_PATH using OP with VALUE.
   Return the created expression, or NULL on error.

   Allowed OP values are CEELOG_EO_{EQ,NE,MATCHES,DOESNT_MATCH}.
   If an expression doesn't contain FIELD_PATH, the expression is false
   regardless of the value of OP. */
extern struct ceelog_expr *ceelog_expr_new_comparison (const char *field_path,
						       enum ceelog_expr_op op,
						       const char *value,
						       GError **error);

/* Create a regex expression that matches a non-JSON event against REGEX
   using OP.
   Return the created expression, or NULL on error.

   Allowed OP values are CEELOG_EO_{MATCHES,DOESNT_MATCH}.
   Note that this is intended to be used only for unstructured events, but they
   may also be expressed in a structured representation; in that case, the
   regex is used to match the "msg" field. */
extern struct ceelog_expr *ceelog_expr_new_unstructured_regex
	(enum ceelog_expr_op op, const char *regex, GError **error);

/* Create a binary expression for OP and subexpressions E1 and E2.
   Return the created expression.
   Allowed OP values are CEELOG_EO_{AND,OR}.
   Takes ownership of E1 and E2, the caller must not free them. */
extern struct ceelog_expr *ceelog_expr_new_binary (enum ceelog_expr_op op,
						   struct ceelog_expr *e1,
						   struct ceelog_expr *e2);

/* Create an unary expression for OP and subexpression SUB.
   Return the created expression.
   The only allowed OP value is CEELOG_EO_NOT.
   Takes ownership of SUB, the caller must not free it. */
extern struct ceelog_expr *ceelog_expr_new_unary (enum ceelog_expr_op op,
						  struct ceelog_expr *sub);

/* Create an expression based on a text EXPR.
   Return the created expression, or NULL on error. */
extern struct ceelog_expr *ceelog_expr_parse (const char *expr, GError **error);


 /* Expression evaluation */

/* A "compiled" ceelog expression, ready for evaluation. */
struct ceelog_compiled_expr;

struct ceelog_event;

/* Compile EXPR.
   Return the compiled expression, or NULL on error. */
extern struct ceelog_compiled_expr *ceelog_compile_expr
	(struct ceelog_expr *expr, GError **error);

/* Parse and compile text EXPR.
   Return the compiled expression, or NULL on error. */
extern struct ceelog_compiled_expr *ceelog_parse_and_compile_expr
	(const char *expr, GError **error);

/* Free a compiled EXPR. */
extern void ceelog_compiled_expr_free (struct ceelog_compiled_expr *expr);

/* Evaluate EXPR on EVENT, return a boolean result.
   On evaluation error, return false. */
extern bool ceelog_eval_expr (struct ceelog_compiled_expr *expr,
			      struct ceelog_event *event);


G_END_DECLS

#endif
